<?php

/**
 * SAML 2.0 IdP configuration for SimpleSAMLphp.
 *
 * See: https://simplesamlphp.org/docs/stable/simplesamlphp-reference-idp-hosted
 */

function nest($name, $value) {
  $element = array_pop($name);
  if ($element) return nest($name, array($element => $value));
  else return $value;
}

$config = SimpleSAML\Configuration::getInstance();
$db_dsn = $config->getString('database.dsn', null);
$db_user = $config->getString('database.username', null);
$db_passwd = $config->getString('database.password', null);
$db = new PDO($db_dsn, $db_user, $db_passwd);

$vserver = $_SERVER['SERVER_NAME'];
$vparts = explode('.', $vserver);
$vhost = $vparts[0];

$raw_users = json_decode(file_get_contents('/opt/simplesamlphp/config/logins.json'), true);
$domains = [];
if (is_array($raw_users)) {
  foreach($raw_users as $user => $values) {
    $sho = @$values['schacHomeOrganization'];
    if ($sho) $domains[] = $sho;
  }
}
$domains = array_unique($domains);

$metadata['__DYNAMIC:1__'] = [
    /*
     * The hostname of the server (VHOST) that will use this SAML entity.
     *
     * Can be '__DEFAULT__', to use this entry by default.
     */
    'host' => '__DEFAULT__',

    'OrganizationName' => $vhost . ' IdP',
    'OrganizationDisplayName' => $vhost . ' IdP',
    'OrganizationURL' => 'https://' . $vserver . '/',

    'contacts' => [
        'a' => [
            'contactType'       => 'technical',
            'emailAddress'      => 'support@'. $vserver,
            'givenName'         => 'John',
            'surName'           => $vhost,
            'telephoneNumber'   => '+31(0)12345678',
            'company'           => $vhost . ' Inc.',
        ],
    ],

    'UIInfo' => array(
        'DisplayName' => array(
            'en' => $vhost . ' IdP'
        ),
        'Description' => array(
            'en' => $vhost . ' IdP description'
        ),
    ),

    'RegistrationInfo' => [
        'authority' => 'urn:mace:' . $vhost,
        'instant' => '2008-01-17T11:28:03Z',
        'policies' => [
            'en' => 'http://' . $vhost . '/policy',
        ],
    ],

    'scope' => $domains,

    // X.509 key and certificate. Relative to the cert directory.
    'privatekey' => 'server.key',
    'certificate' => 'server.crt',

    /*
     * Authentication source to use. Must be one that is configured in
     * 'config/authsources.php'.
     */
    'auth' => 'custom-userpass',
    //'auth' => 'example-userpass',

    /* Uncomment the following to use the uri NameFormat on attributes. */
    //'attributes.NameFormat' => 'urn:oasis:names:tc:SAML:2.0:attrname-format:uri',

    'authproc' => [
	5 => [
          'class' => 'core:TargetedID',
	  'identifyingAttribute' => 'eduPersonTargetedId',
	  'nameId' => TRUE,
	],
	8 => [
	  'class' => 'subjectidattrs:PairwiseID',
	  'identifyingAttribute' => 'pairwise-id',
          'scopeAttribute' => 'scope',
	],
	9 => [
	  'class' => 'subjectidattrs:SubjectID',
	  'identifyingAttribute' => 'subject-id',
          'scopeAttribute' => 'scope',
	],
        15 => [
          'class' => 'core:PHP',
          'code' => '
            unset($attributes["eduPersonTargetedId"]);
            unset($attributes["pairwise-id"]);
            unset($attributes["subject-id"]);
            unset($attributes["scope"]);
          ',
	],
        // Convert LDAP names to urn.
	20 => [
	  'class' => 'core:AttributeMap',
	  'name2oid'
	],
    ],
    'attributes.NameFormat' => 'urn:oasis:names:tc:SAML:2.0:attrname-format:uri',
    'attributeencodings' => array(
        'urn:oid:1.3.6.1.4.1.5923.1.1.1.10' => 'raw', /* eduPersonTargetedID with oid NameFormat. */
    ),
];

$query = "select o.name, o.type, c.value  from idps i
  right join config c on c.idp_id = i.idp_id
  right join options o on c.option_id = o.option_id
  where i.host = :host";


$stmt = $db->prepare($query);
$stmt->execute(array(':host' => $vhost));
$result = $stmt->fetchAll(PDO::FETCH_ASSOC);

$config = array();
foreach ($result as $row) {
  $name = explode(':', $row['name']);
  $value = $row['value'];
  $config = array_merge_recursive($config, nest($name, $value));
}

$metadata['__DYNAMIC:1__'] = array_replace_recursive($metadata['__DYNAMIC:1__'], $config);

