<?php

/**
 * This page serves as a dummy login page.
 *
 * Note that we don't actually validate the user in this example. This page
 * just serves to make the example work out of the box.
 *
 * @package SimpleSAMLphp
 */

if (!isset($_REQUEST['ReturnTo'])) {
    die('Missing ReturnTo parameter.');
}

$returnTo = \SimpleSAML\Utils\HTTP::checkURLAllowed($_REQUEST['ReturnTo']);

/**
 * The following piece of code would never be found in a real authentication page. Its
 * purpose in this example is to make this example safer in the case where the
 * administrator of the IdP leaves the exampleauth-module enabled in a production
 * environment.
 *
 * What we do here is to extract the $state-array identifier, and check that it belongs to
 * the exampleauth:External process.
 */
if (!preg_match('@State=(.*)@', $returnTo, $matches)) {
    die('Invalid ReturnTo URL for this example.');
}

/**
 * The loadState-function will not return if the second parameter does not
 * match the parameter passed to saveState, so by now we know that we arrived here
 * through the customauth:External authentication page.
 */
\SimpleSAML\Auth\State::loadState(urldecode($matches[1]), 'customauth:External');

// our list of users.
$raw_users = json_decode(file_get_contents('/opt/simplesamlphp/config/logins.json'), true);
$attr_raw = json_decode(file_get_contents('/opt/simplesamlphp/config/attributes.json'), true);
foreach($attr_raw as $attribute => $value) {
  $attributes[$attribute] = htmlentities($value, ENT_QUOTES);
}

$users = [];
$explanations = [];
$displays = [];
$categories = [];
foreach ($raw_users as $category => $cat) {
  $categories[$category]['name'] = $category;
  $explanations[$category] = $cat['message'];
  foreach ($cat['profiles'] as $user => $values) {
    $categories[$category]['users'][] = $user;
    $explanations[$user] = $values['explanation'];
    $displays[$user] = $values['display'];
    unset($values['explanation']);
    unset($values['display']);
    $users[$user] = $values;
  }
}
// time to handle login responses; since this is a dummy example, we accept any data
$badUserPass = false;
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (!session_id()) {
        // session_start not called before. Do it here.
        session_start();
    }

    $attributes = @$_POST['keys'] or [];

    // Fix subject scope
    $scope = "dummy.org";
    if (isset($attributes['schacHomeOrganization'])) {
        $scope = $attributes['schacHomeOrganization'];
    } elseif (isset($attributes['eduPersonScopedAffiliation'])) {
        $scope = explode("@", explode(",", $attributes['eduPersonScopedAffiliation'])[0])[1];
    } elseif (isset($attributes['eduPersonPrincipalName'])) {
        $scope = explode("@", $attributes['eduPersonPrincipalName'])[1];
    }
    $attributes['scope'] = $scope;
    //SimpleSAML\Logger::warning("Attributes: " . print_r($attributes, true));

    foreach ($attributes as $key => $value) {
        $_SESSION['attributes'][$key] = $value;
    }

    $username = (string) $_REQUEST['username'];
    if ($username == 'error') {
	$_SESSION['ErrorResponse'] = TRUE;
    }

    \SimpleSAML\Utils\HTTP::redirectTrustedURL($returnTo);
}

// if we get this far, we need to show the login page to the user
$config = \SimpleSAML\Configuration::getInstance();
$t = new \SimpleSAML\XHTML\Template($config, 'customauth:authenticate.tpl.php');
$t->data['users'] = $users;
$t->data['attributes'] = $attributes;
$t->data['explanations'] = $explanations;
$t->data['displays'] = $displays;
$t->data['categories'] = $categories;
$t->data['returnTo'] = $returnTo;
$t->show();
